@@ -2,6 +2,7 @@
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 -- | NOTE: This API is experimental and will probably change. Please try
 -- it out! Feedback is very much appreciated, and your patience in the
 -- face of breaking API changes is also appreciated!
@@ -58,6 +59,7 @@ module Brick.Forms
   , setFieldValid
   , setFormConcat
   , setFieldConcat
+  , setFormFocus
 
   -- * Simple form field constructors
   , editTextField
@@ -65,9 +67,12 @@ module Brick.Forms
   , editPasswordField
   , radioField
   , checkboxField
+  , listField
 
   -- * Advanced form field constructors
   , editField
+  , radioCustomField
+  , checkboxCustomField
 
   -- * Attributes
   , formAttr
@@ -76,16 +81,19 @@ module Brick.Forms
   )
 where
 
-import Graphics.Vty
+import Graphics.Vty hiding (showCursor)
 #if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid
 #endif
 import Data.Maybe (isJust, isNothing)
 import Data.List (elemIndex)
+import Data.Vector (Vector)
 
 import Brick
 import Brick.Focus
+import Brick.Widgets.Core (showCursor)
 import Brick.Widgets.Edit
+import Brick.Widgets.List
 import qualified Data.Text.Zipper as Z
 
 import qualified Data.Text as T
@@ -219,6 +227,10 @@ infixr 5 @@=
     let v = mkFs s
     in v { formFieldRenderHelper = h . (formFieldRenderHelper v) }
 
+-- | Set the focused field of a form.
+setFormFocus :: (Eq n) => n -> Form s e n -> Form s e n
+setFormFocus n f = f { formFocus = focusSetCurrent n $ formFocus f }
+
 -- | Set a form field's concatenation function.
 setFieldConcat :: ([Widget n] -> Widget n) -> FormFieldState s e n -> FormFieldState s e n
 setFieldConcat f s = s { formFieldConcat = f }
@@ -263,7 +275,31 @@ checkboxField :: (Ord n, Show n)
               -> s
               -- ^ The initial form state.
               -> FormFieldState s e n
-checkboxField stLens name label initialState =
+checkboxField = checkboxCustomField '[' 'X' ']'
+
+-- | A form field for manipulating a boolean value. This represents
+-- 'True' as @[X] label@ and 'False' as @[ ] label@. This function
+-- permits the customization of the @[X]@ notation characters.
+--
+-- This field responds to `Space` keypresses to toggle the checkbox and
+-- to mouse clicks.
+checkboxCustomField :: (Ord n, Show n)
+                    => Char
+                    -- ^ Left bracket character.
+                    -> Char
+                    -- ^ Checkmark character.
+                    -> Char
+                    -- ^ Right bracket character.
+                    -> Lens' s Bool
+                    -- ^ The state lens for this value.
+                    -> n
+                    -- ^ The resource name for the input field.
+                    -> T.Text
+                    -- ^ The label for the check box, to appear at its right.
+                    -> s
+                    -- ^ The initial form state.
+                    -> FormFieldState s e n
+checkboxCustomField lb check rb stLens name label initialState =
     let initVal = initialState ^. stLens
 
         handleEvent (MouseDown n _ _ _) s | n == name = return $ not s
@@ -272,7 +308,7 @@ checkboxField stLens name label initialState =
 
     in FormFieldState { formFieldState        = initVal
                       , formFields            = [ FormField name Just True
-                                                            (renderCheckbox label name)
+                                                            (renderCheckbox lb check rb label name)
                                                             handleEvent
                                                 ]
                       , formFieldLens         = stLens
@@ -280,14 +316,60 @@ checkboxField stLens name label initialState =
                       , formFieldConcat       = vBox
                       }
 
-renderCheckbox :: T.Text -> n -> Bool -> Bool -> Widget n
-renderCheckbox label n foc val =
+renderCheckbox :: Char -> Char -> Char -> T.Text -> n -> Bool -> Bool -> Widget n
+renderCheckbox lb check rb label n foc val =
     let addAttr = if foc then withDefAttr focusedFormInputAttr else id
+        csr = if foc then showCursor n (Location (1,0)) else id
     in clickable n $
-       addAttr $
-       (str $ "[" <> (if val then "X" else " ") <> "] ") <+> txt label
+       addAttr $ csr $
+       (txt $ T.singleton lb <> (if val then T.singleton check else " ") <>
+              T.singleton rb <> " ") <+> txt label
 
 -- | A form field for selecting a single choice from a set of possible
+-- choices in a scrollable list. This uses a 'List' internally.
+--
+-- This field responds to the same input events that a 'List' does.
+listField :: forall s e n a . (Ord n, Show n, Eq a)
+          => (s -> Vector a)
+          -- ^ Possible choices.
+          -> Lens' s (Maybe a)
+          -- ^ The state lens for the initially/finally selected
+          -- element.
+          -> (Bool -> a -> Widget n)
+          -- ^ List item rendering function.
+          -> Int
+          -- ^ List item height in rows.
+          -> n
+          -- ^ The resource name for the input field.
+          -> s
+          -- ^ The initial form state.
+          -> FormFieldState s e n
+listField options stLens renderItem itemHeight name initialState =
+    let optionsVector = options initialState
+        initVal = initialState ^. customStLens
+
+        customStLens :: Lens' s (List n a)
+        customStLens = lens getList setList
+            where
+               getList s = let l = list name optionsVector itemHeight
+                           in case s ^. stLens of
+                               Nothing -> l
+                               Just e -> listMoveToElement e l
+               setList s l = s & stLens .~ (snd <$> listSelectedElement l)
+
+        handleEvent (VtyEvent e) s = handleListEvent e s
+        handleEvent _ s = return s
+
+    in FormFieldState { formFieldState        = initVal
+                      , formFields            = [ FormField name Just True
+                                                            (renderList renderItem)
+                                                            handleEvent
+                                                ]
+                      , formFieldLens         = customStLens
+                      , formFieldRenderHelper = id
+                      , formFieldConcat       = vBox
+                      }
+-- | A form field for selecting a single choice from a set of possible
 -- choices. Each choice has an associated value and text label.
 --
 -- This field responds to `Space` keypresses to select a radio button
@@ -301,7 +383,27 @@ radioField :: (Ord n, Show n, Eq a)
            -> s
            -- ^ The initial form state.
            -> FormFieldState s e n
-radioField stLens options initialState =
+radioField = radioCustomField '[' '*' ']'
+
+-- | A form field for selecting a single choice from a set of possible
+-- choices. Each choice has an associated value and text label. This
+-- function permits the customization of the @[*]@ notation characters.
+--
+-- This field responds to `Space` keypresses to select a radio button
+-- option and to mouse clicks.
+radioCustomField :: (Ord n, Show n, Eq a)
+                 => Char
+                 -> Char
+                 -> Char
+                 -> Lens' s a
+                 -- ^ The state lens for this value.
+                 -> [(a, n, T.Text)]
+                 -- ^ The available choices, in order. Each choice has a value
+                 -- of type @a@, a resource name, and a text label.
+                 -> s
+                 -- ^ The initial form state.
+                 -> FormFieldState s e n
+radioCustomField lb check rb stLens options initialState =
     let initVal = initialState ^. stLens
 
         lookupOptionValue n =
@@ -322,7 +424,7 @@ radioField stLens options initialState =
             FormField name
                       Just
                       True
-                      (renderRadio val name label)
+                      (renderRadio lb check rb val name label)
                       (handleEvent val)
 
     in FormFieldState { formFieldState        = initVal
@@ -332,17 +434,18 @@ radioField stLens options initialState =
                       , formFieldConcat       = vBox
                       }
 
-renderRadio :: (Eq a) => a -> n -> T.Text -> Bool -> a -> Widget n
-renderRadio val name label foc cur =
+renderRadio :: (Eq a) => Char -> Char -> Char -> a -> n -> T.Text -> Bool -> a -> Widget n
+renderRadio lb check rb val name label foc cur =
     let addAttr = if foc
                   then withDefAttr focusedFormInputAttr
                   else id
         isSet = val == cur
+        csr = if foc then showCursor name (Location (1,0)) else id
     in clickable name $
-       addAttr $
-       hBox [ str "["
-            , str $ if isSet then "*" else " "
-            , txt $ "] " <> label
+       addAttr $ csr $
+       hBox [ txt $ T.singleton lb
+            , txt $ if isSet then T.singleton check else " "
+            , txt $ T.singleton rb <> " " <> label
             ]
 
 -- | A form field for using an editor to edit the text representation of
